/*---------------------------------------------------------------------------
  This code is part of the book
  Clocksin William F., Mellish Christopher S. (2003). 
  Programming in Prolog: Using the ISO Standard. 5th edition. Springer.
----------------------------------------------------------------------------*/

read_in([W|Ws]):-
	get0(C),
	readword(C, W, C1),
	restsent(W, C1, Ws), !. /*The ! is mine...*/

restsent(W, _, []):-
	lastword(W), !.
restsent(_, C, [W1|Ws]):-
	readword(C, W1, C1),
	restsent(W1, C1, Ws).

readword(C, W, C1):-
	single_character(C), !,
	name(W, [C]),
	get0(C1).
readword(C, W, C2):-
	in_word(C, NewC), !,
	get0(C1),
	restword(C1, Cs, C2),
	name(W, [NewC|Cs]).
readword(_, W, C2):-
	get0(C1),
	readword(C1, W, C2).

restword(C, [NewC|Cs], C2):-
	in_word(C, NewC), !,
	get0(C1),
	restword(C1, Cs, C2).
restword(C, [], C).


single_character(33).	% !
%single_character(35).	% #
single_character(39).	% '
single_character(40).	% (
single_character(41).	% )
single_character(42).	% *
single_character(43).	% +
single_character(44).	% ,
%single_character(45).	% -
single_character(46).	% .
single_character(47).	% /
%single_character(58).	% :
single_character(59).	% ;
%single_character(60).	% <
%single_character(61).	% =
%single_character(62).	% >
single_character(63).	% ?
single_character(91).	% [
single_character(93).	% ]
single_character(94).	% ^
single_character(123).	% {
single_character(124).	% |
single_character(125).	% }

in_word(C, C):-
	C>96, C<123.
in_word(C, L):-
	C>64, C<91,
	L is C+32.
in_word(C, C):-
	C>47, C<58.
in_word(39, 39).
in_word(95, 95).
in_word(35, 35).

in_word(60,60).	% <
in_word(61,61).	% =
in_word(62,62).	% >
in_word(45,45).	% -
in_word(58,58). % :
in_word(92,92). % \

lastword('.').

wordtolist(W,L):- var(W),!, ltow(W,L).
wordtolist(W,L):- var(L),!, wtol(W,L).

wtol(W,L):-
	name(W,ASCII),
	asciitoletter(ASCII,L).

asciitoletter([],[]).
asciitoletter([ASCIIH|ASCIIT],[H|T]):-
	name(H,[ASCIIH]),!,
	asciitoletter(ASCIIT,T).

ltow(W,L):-
	asciitoletter(ASCII,L),
	name(W,ASCII).


myread(S,L):-
	read_in(S),
	(comment(S,S1)->
		L=S1;		
		convert(S,L)).

comment(S,S1):-
	append(X,['*', '/', '.'],S),
	append(['/','*'],_,X),
	append(S1,['.'],S),!.

convert(['.'],[]).
%convert([H|T],[H|R]):-
%	reserved(H),!,
%	convert(T,R).
convert([H|T],[H|R]):-
	tag(H),!,
	convert(T,R).
convert([H|T],List):-
	convert(T,R),
	wordtolist(H,W),
	conc(W,R,List).

conc([],L,L).
conc([H|T],L,[H|R]):-
	conc(T,L,R).


