/*---------------------------------------------------------------------------
  This code is part of the book
  LOGIC PROGRAMMING TECHNIQUES: THE PROLOG LANGUAGE
  Ilias Sakellariou, Nikos Bassiliades, Petros Kefalas, Demosthenes Stamatis
  Hellenic Academic On-line Publications, 2015
----------------------------------------------------------------------------*/

/*------------------------------------------------------------------
               KNOWLEDGE REPRESENTED AS A FRAME STRUCTURE
-------------------------------------------------------------------*/

frame(thing).
frame(mammal).
frame(plant).
frame(fruit).
frame(banana).
frame(monkey).
frame(tree).
frame(cat).
frame(dog).
frame(human).
frame(man).
frame(woman).

frame_prop(mammal, isa, thing).
frame_prop(plant, isa, thing).
frame_prop(fruit, isa, thing).
frame_prop(banana, isa, fruit).
frame_prop(monkey, isa, mammal).
frame_prop(human, isa, mammal).
frame_prop(man, isa, human).
frame_prop(woman, isa, human).
frame_prop(cat, isa, mammal).
frame_prop(dog, isa, mammal).
frame_prop(tree, isa, plant).

frame_prop(monkey, likes, banana).
frame_prop(man, likes, woman).
frame_prop(mammal, eats, fruit).
frame_prop(banana, edible, true).
frame_prop(dog, hates, cat).
frame_prop(human, enjoys, hunting).

/*------------------------------------------------------------------
               RETRIEVING PROPARTIES FROM FRAME HIERARCY
-------------------------------------------------------------------*/
property(O,A,V):-
	frame_prop(O,A,V),!.
property(O,A,V):-
	frame_prop(O,isa,SuperClass),!,
	property(SuperClass, A, V),!.

/*------------------------------------------------------------------
               IF THEN RULES DEFINING SEMANTICS
-------------------------------------------------------------------*/
	
eligible_action(X,eats,Y,[X,isa,mammal]+[X,likes,Y]+[Y,edible,true]):-
	property(X,isa,mammal),
	property(X,likes,Y),
	property(Y,edible,true).
	
eligible_action(X,chases,Y,[X,isa,mammal]+[Y,isa,mammal]+[X,likes,Y]+[X,enjoys,hunting]):-
	property(X,isa,mammal),
	property(Y,isa,mammal),
	property(X,likes,Y),
	property(X,enjoys,hunting).
	

eligible_action(X,chases,Y,[X,isa,mammal]+[X,hates,Y]):-
	property(X,isa,mammal),
	property(X,hates,Y).

/*------------------------------------------------------------------
               CHECKING FOR SEMANTICALLY CORRRECT SENTENCES
-------------------------------------------------------------------*/
semantics(S):-
	S=[_,Object1,Verb,_,Object2],
	sentence(S, []),
	eligible_action(Object1, Verb, Object2, Justification),
	writelist(['Sentence: ',S,' is semantically correct',nl]),
	writelist([Justification,nl,nl]).

/*------------------------------------------------------------------
               A SIMPLE LANGUAGE GRAMMAR
-------------------------------------------------------------------*/
sentence --> noun_phrase, verb_phrase.

noun_phrase-->article,noun.

verb_phrase-->verb,noun_phrase.

noun --> [cat].
noun --> [dog].
noun --> [man].
noun --> [woman].
noun --> [monkey].
noun --> [banana].
noun --> [tree].

article --> [a].
article --> [the].

verb  --> [chases].
verb --> [eats].

/*------------------------------------------------------------------
               AUXILIARY
-------------------------------------------------------------------*/
writelist([]):-!.
writelist([nl|T]):-!,
	nl,
	writelist(T).
writelist([H|T]):-!,
	write(H),
	writelist(T).